 
;****************************************************
;
;******************* GARAGE RECEIVER MAILBOX *****************
;
;Written by Ronald R. Newton Copyrighted FEBRUARY 6,2011
;****************************************************
  
;REV A
;02/06/2011
;XTAL 4 MHz
;USE RS232 1200 BAUD 
;USES WATCH DOG TIMER AS A COUNTER FOR FLASHING LED

;PIN 1 VCC
;PIN 2 RB5 	  NC   
;PIN 3 RB4    VOLTAGE DETECTOR                 
;PIN 4 RB3    PUSH BUTTON
;PIN 5 RC5    PDN     
;PIN 6 RC4	  T/R SELECT
;PIN 7 RC3	  DATA
;PIN 8 RC2	  SPEAKER
;PIN 9 RC1    LED COMMON
;PIN 10 RC0   NC
;PIN 11 RB2	  LED 3
;PIN 12 RB1   LED 2
;PIN 13 RB0   LED 1
;PIN 14 VSS


	RADIX		DEC	;DECIMAL IS DEFAULT # FORMAT
    list      p=16F505            ; list directive to define processor
    #include <p16F505.inc>        ; processor specific variable definitions

    __CONFIG   _MCLRE_OFF & _CP_OFF & _WDT_ON & _IntRC_OSC_RB4EN
 
DELAY_CNTR     	EQU		008H		;COUNTING REGISTER FOR RS232
BIT_CNTR       	EQU     009H       	;COUNTING REGISTER FOR RS232
BIT_CNTR1     	EQU     00AH       	;COUNTING REGISTER FOR RS232
XMT_BYTE       	EQU     00BH       	;COUNTING REGISTER FOR RS232
XMT_BYTE1      	EQU     00CH       	;TRANSFER REGISTER FOR RS232
KEY1      		EQU     00DH       	;KEY 1 FOR RECEIVER
KEY2			EQU	   	00EH		;KEY 2 FOR RECEIVER
FLAGS			EQU		00FH		;GENERAL FLAG REGISTER
COUNTER			EQU		010H		;COUNTING REGISTER
COUNTER1		EQU		011H		;COUNTING REGISTER
COUNTER2		EQU		012H		;COUNTING REGISTER
COUNTER3		EQU		013H		;COUNTING REGISTER
LOOP1			EQU		014H		;COUNTING REGISTER
DELAY_CNTR1		EQU		015H		;COUNTING REGISTER
RCV_BYTE		EQU		016H		;RECEIVED FIRST BYTE FROM DATA 
KEY2TRANS		EQU		017H		;RECEIVED SECOND BYTE FROM DATA 
KEY1TRANS		EQU		018H		;RECEIVED FIRST BYTE FROM DATA
RCV_BYTE1		EQU		019H		;RECEIVED SECOND BYTE FROM DATA
TEMP			EQU		01AH		;TEMPERARY HOLDING REGISTER
RESULTS			EQU		01BH		;RESULTS OF FIRST 3 BITS KEY 2
COUNTER5		EQU		01CH		;COUNTING REGISTER
LOOP2			EQU		01DH		;COUNTING REGISTER
LOOP3			EQU		01EH		;COUNTING REGISTER
LOOP4			EQU		01FH		;COUNTING REGISTER


#DEFINE		   	LED1		PORTB,0			;LED 1
#DEFINE		   	LED2		PORTB,1			;LED 2
#DEFINE			LED3		PORTB,2			;LED 3
#DEFINE		 	PUSH		PORTB,3			;ALARM SWITCH
#DEFINE			VD  		PORTB,4			;VOLTAGE DETECTOR
#DEFINE			COMMON		PORTC,0			;COMMON LED ANODE
#DEFINE		    SPK			PORTC,2			;SPEAKER
#DEFINE		   	T_R			PORTC,4			;TRANSMIT OR RECEIVE
#DEFINE		   	PDN			PORTC,5			;ACTIVATION OF TRANSMITTER
#DEFINE			ALARM_FLAG  FLAGS,0			;ALARM FLAG
#DEFINE			DATA_IO		PORTC,3			;DATA LINE
#DEFINE			FLAG1		FLAGS,1

	ORG	00H			;STARTING AND RESET LOCATION

MAIN    	MOVWF   	OSCCAL            	; update register with factory cal value 

;***************THIS SECTION LOADS COMBINATION CODE INTO KEY 1 AND KEY 2*************

;********INPUT KEY 1 IN HEXIDECIMAL E.G.0A1H
			MOVLW		0AAH		;INPUT KEY NUMBER HERE
			MOVWF		KEY1		;LOAD

;********INPUT LAST 5 BITS KEY 2 IN HEXIDECIMAL E.G., 0A0H  (ZERO MUST PRECEED ANY LETTER)
			MOVLW		0F8H		;INPUT KEY NUMBER HERE (0A8H) OR (AOH)  FIRST THREE BITS IGNORED
 		    MOVWF		KEY2		;LOAD

;**********************SETTING UP MICRO PORTS*********************
AWAYWEGO
		CLRF		PORTB			;CLEAR PORT B
		MOVLW		B'011000'		;INPUT FOR TRISB
		TRIS		PORTB			;LOAD TRISB
   	  	CLRF		PORTC			;CLEARING PORT C
	  	MOVLW		B'001000'		;INPUT FOR TRISC
	  	TRIS		PORTC			;LOAD
		MOVLW		B'00001110'	    ;INPUT FOR OPTION WAKE UP ON PIN CHANGE USE PULLUPS
		OPTION						;LOAD OPTION
		BCF			COMMON			;TURN OFF LEDS
		BCF			LED1   			;TURN OFF LEDS
		BCF			LED2			;TURN OFF LEDS
		BCF			LED3			;TURN OFF LEDS
		BCF			T_R				;MAKE LINX RECEIVER
		BCF			PDN				;TURN ON LINX


;*****************THIS AREA IS FOR TESTING PURPOSES*****************************
TESTING_AREA 
	
;***************THIS AREA DISPLAYS LEDS ABOUT EVERY 5 SECONDS******************
TIMER_AREA
		BTFSC		PUSH			;PUSHED FOR ALARMING
		BCF			ALARM_FLAG		;CLEAR FLAG

		MOVLW		.4 ^0FFH		;CHECK TO SEE IF COUNTER 1 IS ABOVE 4
		ADDWF		COUNTER1,0		
		BTFSS		3,0
		GOTO		TA1				;NOT ABOVE GO TO TA1
		MOVLW		.4				;WAS ABOVE SET TO 4
		MOVWF		COUNTER1		;LOAD
TA1	    DECFSZ		COUNTER1,1		;DECREASE 4 TIMES
		GOTO		CBV				;JUMP TO VOLTAGE CHECK
		MOVLW		.4				;LOAD 4 INTO COUNTER 1
		MOVWF		COUNTER1		;LOAD
		GOTO		BLINK			;ABOUT 5 SECONDS BLINK

;***************CHECKING BATTERY VOLTAGE *******************
CBV	  ;  BTFSS		VD				;CHECK VOLTAGE
	;	CALL		SPEAKER			;VOLTAGE LOW CALL SPEAKER
	;	MOVLW		.10				;LOADING LOOP 2 WITH 10 
	;	MOVWF		LOOP2			;LOAD

;*************THIS AREA ELIMINATES FALSE SIGNALS******************
RESET	CLRF		COUNTER2 		;CLEARING TIMER FOR CHECK
		MOVLW		.5				;LOADING TIMER FOR CHECK
		MOVWF		COUNTER3		;LOAD
		DECFSZ		LOOP1,1			;ALLOW TO RUN 20 MICROSECONDS
		GOTO		CHECK			;ALLOW TO RUN 20 MICROSECONDS
		DECFSZ		LOOP2,1			;ALLOW TO RUN 20 MICROSECONDS
		GOTO		CHECK			;ALLOW TO RUN 20 MICROSECONDS
		BCF			PDN				;TURN OFF LINX
		SLEEP						;PUT TO SLEEP
		NOP							;FOR DEBUGGING

	
CHECK   BSF			PDN 	
		BTFSS		DATA_IO			;MAKE SURE DATA IN IS HIGH FOR A PERIOD OF TIME INDICATING TRANSMITTER SIGNAL
	  	GOTO		RESET			;NOT HIGH CHECK AGAIN		
		DECFSZ		COUNTER2,1		;TIMER
		GOTO		CHECK			;NOT HIGH CHECK AGAIN
		DECFSZ		COUNTER3,1		;TIMER
		GOTO		CHECK			;TIMER
		

C0	  	BTFSC		DATA_IO			;CHECK FOR START SIGNAL FROM TRANSMITTER
		GOTO		C0				;NO DATA GO TO CHECK
C1		CALL		START_DELAY		;START PULSE CHECKING TO SEE IF VALID
		BTFSC		DATA_IO			;STILL LOW?
		GOTO		C0				;START OVER AGAIN
	
	    
;********************READING DATA *****************************
		MOVLW		.8				;LOAD BIT COUNTERS
		MOVWF		BIT_CNTR		;LOAD
		MOVWF		BIT_CNTR1		;LOAD
		CLRF		RCV_BYTE		;CLEAR RCV BYTE
		CLRF		RCV_BYTE1		;CLEAR RCV BYTE 1
NEXT_RCVBIT							;LABEL
		CALL		BIT_DELAY		;CALLING BIT DELAY
		BTFSS		DATA_IO			;IS DATA IN HIGH?
		BCF			STATUS,C		;NO CLEAR STATUS C
		BTFSC		DATA_IO			;IS DATE LOW?
		BSF			STATUS,C		;NO SET STATUS C
		RRF			RCV_BYTE,1		;ROTATE RECEIVE BYTE OUT
		DECFSZ		BIT_CNTR,1		;CHECK TO SEE IF 8 BIT SENT
		GOTO		NEXT_RCVBIT		;NOT 8 DO AGAIN

NB		MOVF		RCV_BYTE,W		;DONE MOVE RECEIVE BYTE INTO KEY1TRANS
		MOVWF		KEY1TRANS		;LOAD

NEXT_RCVBIT1						;LABEL
		CALL		BIT_DELAY		;CALL BIT DELAY
		BTFSS		DATA_IO			;CHECK SERIAL IN
		BCF			STATUS,C		;LOW CLEAR STATUS C
		BTFSC		DATA_IO			;CHECK SERIAL IN
		BSF			STATUS,C		;HIGH SET STATUS C
		RRF			RCV_BYTE1,1		;ROTATE BIT OUT
		DECFSZ		BIT_CNTR1,1		;CHECK FOR 8 BITS
		GOTO		NEXT_RCVBIT1	;NOT 8 BITS DO AGAIN
		CALL		BIT_DELAY		;CALL BIT DELAY
		MOVF		RCV_BYTE1,W		;MOVING RECEIVE BYTE INTO KEY2TRANS
		MOVWF		KEY2TRANS		;LOAD
		GOTO		DECODE			;GO TO DECODE
      
;********************RS232 INSTRUCTIONS***********

START_DELAY							;LABEL
		MOVLW		D'100'		   	;SETUP FOR 1200 BAUD SETS FOR DETECTING MIDDLE OF BIT .200
		MOVWF		DELAY_CNTR	   	;DELAY LOOP
LOOP_2	NOP							;DELAY HELP
		DECFSZ		DELAY_CNTR,1	;DECREASEING COUNTER
	    GOTO		LOOP_2          ;DECREASE COUNTER
        RETLW 		0              	;RETURN TO CALL
 
BIT_DELAY 
		MOVLW	D'200'			   	;GENERATING 1200 BAUD DELAY
		MOVWF	DELAY_CNTR         	;LOAD INTO COUNTER
LOOP    NOP						   	;TIMEING HELPER
	    DECFSZ	DELAY_CNTR,1			;DECREMENT TIMER
		GOTO	LOOP		       	;DECREASE COUNTER
        RETLW   0                   ;RETURN TO CALL 
DECODE								;LABEL
		BCF			PDN
;*****************CHECKING KEY ONE***************
		MOVF		KEY1,0			;MOVE KEY1 INTO W
		SUBWF		KEY1TRANS,0		;SUBTRACT TRANSMITTED KEY
		BTFSC		3,2				;CHECK TO SEE IF IDENTICAL
		GOTO		D1				;IDENTICAL GO TO D1
		GOTO		RESET			;DID NOT MATCH START OVER

;**************ELIMINATING KEY TWO FIRST 3 BITS ***************
D1		MOVF		KEY2TRANS,0		;MAKE A DUPLICATE OF KEY 2 TRANSMISSION	
		MOVWF		TEMP			;LOAD INTO TEMP
		MOVLW		B'11111000'		;GETTING RID OF FIRST 3 DIGITS OF KEY 2
		ANDWF		KEY2TRANS,1		;GETTING RID OF FIRST 3 DIGITS OF KEY 2
		MOVLW		B'00000111'		;GETTING RID OF FIRST 3 DIGITS OF TEMP
		ANDWF		TEMP,1			;GETTING RID OF FIRST 3 DIGITS OF TEMP

;****************CHECKING KEY TWO****************
		MOVF		KEY2,0			;MOVE KEY2 INTO W
		SUBWF		KEY2TRANS,0		;SUBTRACT TRANSMITTED KEY DUPLICATE
		BTFSC		3,2				;CHECK TO SEE IF IDENTICAL
		GOTO		D2				;IDENTICAL GOT TO D2
		GOTO		RESET			;NOT IDENTICAL START OVER
D2		MOVF		TEMP,0			;LOADING REGISTER INTO RESULTS
		MOVWF		RESULTS			;LOAD
	
;*****************THIS AREA PROGRAMS LEDS  ****************
BLINK	
		BTFSC		ALARM_FLAG		;CHECK PUSH SWITCH
		GOTO		GD1A		
		BTFSC		RESULTS,2		;CHECK RESULTS 2
		GOTO		BLINK1
		BCF			FLAG1			;LOW CLEAR FLAG
		GOTO		GD1A
BLINK1	BSF			FLAG1			;HIGH SET FLAG

;******************DISPLAYING PUSH SWITCH SETTING ************************
GD1A	BTFSC		PUSH			;CHECK TO SEE IF SET OR CLEAR
		GOTO		GD1B			;SET, GO TO GD1B
		BCF			LED1			;NOT SET, MAKE LED 1 - GREEN
		BSF			COMMON			;MAKE COMMON +
		MOVLW		B'011010'		;TURN OFF LED 2
		TRIS		PORTB			;TURN OFF LED2
		BCF			LED3			;TURN ON GREEN LED3
		CALL		FLASH			;FLASH LED 1
		GOTO		GD2A			;JUMP TO NEXT LED
GD1B	BSF			LED1			;NOT SET, MAKE LED1 +
		BCF			COMMON			;MAKE COMMON -
		MOVLW		B'011010'		;TURN OFF LED 2
		TRIS		PORTB			;TURN OFF LED2
		BSF			LED3			;TURN OFF LED 3
		CALL		FLASH			;FLASH LED 1

;***************** DISPLAYING MAILBOX **************************
GD2A	BTFSC		FLAG1			;CHECK TO SEE IF SET OR CLEAR
		GOTO		GD2B			;SET, GO TO BD2B 
		BCF			LED2			;NOT SET MAKE LED 2 -
		BSF			COMMON			;MAKE COMMON +
		MOVLW		B'011101'		;TURN OFF LED 2
		TRIS		PORTB			;TURN OFF LED2
		CALL		FLASH			;FLASH LED 2
		GOTO		CLEAR			;JUMP TO NEXT LED
GD2B	BSF			LED2			;NOT SET, MAKE LED2 +
		BCF			COMMON			;MAKE COMMON -
		MOVLW		B'011101'		;TURN OFF LED 1 & 3
		TRIS		PORTB			;TURN OFF LED 1 & 3
		CALL 		FLASH			;FLASH LED 2


;******************TURNING OFF ALL LEDS*******************
CLEAR	BCF			COMMON			;TURNING ALL OFF LEDS
		BCF			LED1
		BCF			LED2
		BCF			LED3
;***************CHECKING ALARM SWITCH ***************
SWITCH	BTFSS		PUSH			;CHECK ALARM SWITCH
		GOTO		SIREN			;ALARM SWITCH ON JUMP TO SIREN
		GOTO		RESET			;NOT ON GO TO CHECK


;***************TIMING FOR FLASHING LEDS***********
FLASH 	CLRWDT						;TURN OFF WATCH DOG TIMER
		MOVLW		100				;MOVE 3 INTO LOOP4
		MOVWF		LOOP3			;LOAD
F1	 	DECFSZ		LOOP2,1			;TIMING FOR FLASH
		GOTO		F1				;TIMING FOR FLASH
		DECFSZ		LOOP3,1			;TIMING FOR FLASH
		GOTO		F1				;TIMING FOR FLASH
		RETLW		0				;RETURN TO CALL

;**************SPEAKER GENERATOR******************
SPEAKER	
		CLRWDT						;TURN OFF WDT
		MOVLW		.068			;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BCF			SPK				;TURN ON SPEAKER
		DECFSZ		LOOP1,1			;TURN ON FOR A PERIOD OF TIME
		GOTO		$-1				;TURN ON FOR A PERIOD OF TIME
		MOVLW		.068			;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BSF			SPK				;TURN OFF SPEAKER
		DECFSZ		LOOP1,1			;TURN OFF FOR A PERIOD OF TIME
		GOTO		$-1				;TURN OFF FOR A PERIOD OF TIME
		DECFSZ		LOOP2,1			;SOUND SPEAKER FOR A PERIOD OF TIME	
		GOTO		SPEAKER			;SOUND SPEAKER FOR A PERIOD OF TIME
		RETLW		0				;RETURN TO CALL

;***************ALARM AREA FOR INTRUDER**************************	
SIREN	BTFSS		FLAG1		;CHECK FLAG1
		GOTO		SI0			;HIGH SOUND ALARM
		GOTO		RESET		;ALL LOW GO TO 3

SI0		BSF			ALARM_FLAG  ;SET ALARM FLAG
		CLRWDT					;TURN OFF WDT
		MOVLW		.002		;MOVE DECIMAL 2 INTO W
		MOVWF		LOOP3		;LOAD IN LOOP3
SI1		MOVLW		.186		;MOVE DECIMAL 186 INTO W
		MOVWF		LOOP1		;LOAD IN LOOP1
		BCF			SPK			;TURN ON SPEAKER
		DECFSZ		LOOP1,1		;DECREASE LOOP 1 UNTIL ZERO
		GOTO		$-1			;JUMP IF NOT ZERO BACK ONE
		MOVLW		.186		;MOVE DECIMAL 186 INTO W
		MOVWF		LOOP1		;LOAD INTO LOOP1
		BSF			SPK			;TURN OFF SPEAKER
		DECFSZ		LOOP1,1		;DECREASE LOOP 1 UNTIL ZERO
		GOTO		$-1			;JUMP IF NOT ZERO BACK ONE	
		DECFSZ		LOOP2,1		;DECREASE LOOP 2 (256) UNTIL ZERO
		GOTO		SI1			;NOT ZERO GO BACK TO SI1
		DECFSZ		LOOP3,1		;DECREASE LOOP 3 UNTIL ZERO
		GOTO		SI1			;IF NOT ZERO GOTO SI1 (TIME DELAY FOR TONE 1 LOW)
		MOVLW		.004		;MOVE DECIMAL 4 INTO W
		MOVWF		LOOP3		;LOAD IN LOOP 3
SI2		MOVLW		.068		;MOVE DECIMAL 68 INTO W
		MOVWF		LOOP1		;LOAD INTO LOOP 1
		BCF			SPK			;TURN ON SPEAKER
		DECFSZ		LOOP1,1		;DECREASE LOOP 1 UNTIL ZERO 
		GOTO		$-1			;JUMP IF NOT ZERO BACK ONE
		MOVLW		.068		;MOVE DECIMAL 68 INTO W
		MOVWF		LOOP1		;LOAD INTO LOOP 1
		BSF			SPK			;TURN OFF SPEAKER
		DECFSZ		LOOP1,1		;DECREASE LOOP 1 UNTIL ZERO
		GOTO		$-1			;JUMP IF NOT ZERO BACK ONE
		DECFSZ		LOOP2,1		;DECREASE LOOP 2 UNTIL ZERO
		GOTO		SI2			;IF NOT ZERO GOT SI2
		DECFSZ		LOOP3,1		;DECREASE LOOP 3 UNTIL ZERO
		GOTO		SI2			;IF NOT ZERO GOTO SI2 (TIME DELAY FOR TONE 2 HIGH)
SI3		GOTO		RESET		;GO TO CHECK     
		  

		END
