 
;****************************************************
;
;******************* GARAGE TRANSMITTER *****************
;
;WRITTEN BY RONALD R. NEWTON COPYRIGHTED APRIL 10 2010
;****************************************************
  
;REVISION D  REV APRIL 8 2011  ADDED WARM UP TIMER 
;03/27/2010 
;XTAL 4 MHz
;USE RS232 1200 BAUD 
;USES WATCH DOG TIMER FOR CHECKING BATTERY VOLTAGE

;PIN 1 VCC
;PIN 2 RB5 	  NC   
;PIN 3 RB4    TILT SWITCH                 
;PIN 4 RB3    VOLT DETECTOR
;PIN 5 RC5    PDN     
;PIN 6 RC4	  T/R SELECT
;PIN 7 RC3	  DATA
;PIN 8 RC2	  SPEAKER
;PIN 9 RC1    NC
;PIN 10 RC0   NC
;PIN 11 RB2	  NC
;PIN 12 RB1   SWITCH 2
;PIN 13 RB0   SWITCH 1
;PIN 14 VSS


	RADIX		DEC	;DECIMAL IS DEFAULT # FORMAT
    list      p=16F505            ; list directive to define processor
    #include <p16F505.inc>        ; processor specific variable definitions

    __CONFIG   _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC_RB4EN
 
DELAY_CNTR     	EQU		008H	  	;COUNTING REGISTER FOR RS232
BIT_CNTR       	EQU     009H       	;COUNTING REGISTER FOR RS232
BIT_CNTR1     	EQU     00AH       	;COUNTING REGISTER FOR RS232
XMT_BYTE       	EQU     00BH       	;COUNTING REGISTER FOR RS232
XMT_BYTE1      	EQU     00CH       	;TRANSFER REGISTER FOR RS232
KEY1      		EQU     00DH       	;KEY 1 FOR RECEIVER
KEY2			EQU	   	00EH	  	;KEY 2 FOR RECEIVER
LOOP1			EQU		00FH	  	;COUNTING REGISTER FOR TONE GENERATION
COUNTER			EQU		010H	  	;COUNTING REGISTER FOR TRANSMITTER
COUNTER1		EQU		011H	  	;COUNTING REGISTER
COUNTER2		EQU		012H	  	;COUNTING REGISTER	
TONE_TIMER		EQU		013H	  	;COUNTING REGISTER	
TIMER1			EQU		014H
TIMER2			EQU		015H
	


#DEFINE		   	S1			PORTB,0			;SWITCH 1
#DEFINE		   	S2			PORTB,1			;SWITCH 2
#DEFINE			TILT		PORTB,4			;TILT SWITCH OR SWITCH3
#DEFINE			VD			PORTB,3			;VOLTAGE DETECTOR
#DEFINE		    SPK			PORTC,2			;SPEAKER
#DEFINE		   	SERIAL_OUT 	PORTC,3     	;DATA
#DEFINE		   	T_R			PORTC,4			;TRANSMIT OR RECEIVE
#DEFINE		   	PDN			PORTC,5			;ACTIVATION OF TRANSMITTER
#DEFINE			SERIAL_IN	PORTC,3			;DATA IN



	ORG	00H			;STARTING AND RESET

MAIN  	  	MOVWF   	OSCCAL            	; update register with factory cal value 

;**************THIS SECTION LOADS COMBINATION CODE INTO KEY 1 AND KEY 2 **************************

;********INPUT KEY 1 IN HEXIDECIMAL E.G.0A1H
			MOVLW		0AAH		;INPUT KEY NUMBER HERE
			MOVWF		KEY1		;LOAD

;********INPUT LAST 5 BITS KEY 2 IN HEXIDECIMAL E.G., 0A0H  (ZERO MUST PRECEED ANY LETTER)
			MOVLW		0F8H		;INPUT KEY NUMBER HERE (0A8H) OR (AOH)  FIRST THREE BITS IGNORED
 		    MOVWF		KEY2		;LOAD

;************THIS SECTION LOADS PORTS AND OPTION REGISTER *******************
AWAYWEGO							;LABEL
			CLRF		PORTB		;CLEAR PORT B
			MOVLW		B'011011'	;INPUT FOR TRISB
			TRIS		PORTB		;LOAD TRISB
    	  	CLRF		PORTC		;CLEARING PORT C
		  	MOVLW		B'000000'	;INPUT FOR TRISC
		  	TRIS		PORTC		;LOAD
			MOVLW		B'00001111' ;INPUT FOR OPTION WAKE UP ON PIN CHANGE USE PULLUPS
			OPTION					;LOAD OPTION
			BSF			SERIAL_OUT	;MAKE SERIAL_OUT HIGH

;***********************THIS AREA IS USED FOR PROGRAM TESTING************
TESTING_AREA

;**********THIS SECTION CHECKS TO SEE WHAT WOKE UP MICRO***************8
			BSF		T_R				;MAKE TRANSMITTER
			BSF		PDN				;TURN ON LINX
			DECFSZ	TIMER1			;DELAY TIMER FOR WARMUP
			GOTO	$-1				;DELAY TIMER FOR WARMUP
			DECFSZ	TIMER2			;DELAY TIMER FOR WARMUP
			GOTO	$-3				;DELAY TIMER FOR WARMUP
			BCF		PDN				;TURN OFF LINX
		    BTFSC		3,7			;CHECKING STATUS FOR PIN CHANGE
			GOTO		PREPARE		;PIN CHANGE GO TO PIN CHANGE (PC)
		    BTFSS		VD			;WDT TIMED OUT CHECK VOLTAGE
			CALL		SPEAKER		;VOLTAGE LOW CALL SPEAKER
			BCF			PDN			;TURN OFF LINX
			MOVF		PORTB,0
			SLEEP					;PUT TO SLEEP

;****************PREPARING KEYS TO SEND******************
PREPARE 	BCF		STATUS,7		;CLEAR STATUS BIT 7 ADDED DEC 4 2010
			BSF		PDN				;TURN ON LINX
			BSF		T_R				;MAKE IT A TRANSMITTER

			MOVLW	.10				;RESET TIMER
			MOVWF	COUNTER1		;LOAD
P1			DECFSZ	COUNTER,1		;TIMER FOR POWER UP DELAY
			GOTO	P1				;TIMER PROGRAM
			DECFSZ	COUNTER1,1		;TIMER PROGRAM
			GOTO	P1				;TIMER PROGRAM

P2			BSF		KEY2,0			;SET KEY2,0
			BSF		KEY2,1			;SET KEY2,1
			BSF		KEY2,2			;SET KEY2,2
			BTFSS	S1				;CHECK S1				;PUT A ";" IN FRONT OF BTFSS TO MAKE S1 ALWAYS LOW
			BCF		KEY2,0			;LOW CLEAR KEY2,0
			BTFSS	S2				;CHECK S2				;PUT A ";" IN FRONT OF BTFSS TO MAKE S2 ALWAYS LOW
			BCF		KEY2,1			;LOW CLEAR KEY2,1
			BTFSS	TILT			;CHECK TILT SWITCH
			BCF		KEY2,2			;TILT SWITCH CLEAR KEY2,0
			MOVF	KEY1,0			;LOAD KEY1 INTO TRANSMIT BYTE
			MOVWF	XMT_BYTE		;LOAD
			MOVF	KEY2,0			;LOAD KEY2 INTO TRANSMIT BYTE 1
	    	MOVWF	XMT_BYTE1		;LOAD
			CLRWDT					;CLEAR WATCH DOG TIMER
			CALL 	SEND			;CALL SEND
			DECFSZ	COUNTER2,1		;SEND FOR 6 SECONDS
			GOTO	PREPARE			;JUMP TO PREPARE
			BCF		PDN				;TURN OFF LINX
			MOVF	PORTB,0			;READ PINS BEFORE SLEEP
			SLEEP					;GO TO SLEEP


;********************** ALARM FOR LOW VOLTAGE ********************
SPEAKER								;LABEL
		CLRWDT						;TURN OFF WDT
		MOVLW		.068			;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BCF			SPK				;TURN ON SPEAKER
		DECFSZ		LOOP1,1			;TURN ON FOR A PERIOD OF TIME
		GOTO		$-1				;TURN ON FOR A PERIOD OF TIME
		MOVLW		.068			;LOADING NUMBER
		MOVWF		LOOP1			;LOADING NUMBER INTO LOOP1
		BSF			SPK				;TURN OFF SPEAKER
		DECFSZ		LOOP1,1			;TURN OFF FOR A PERIOD OF TIME
		GOTO		$-1				;TURN OFF FOR A PERIOD OF TIME
		DECFSZ		TONE_TIMER,1	;MAKE NOISE FOR 100 MILLI SECONDS
		GOTO		SPEAKER			;DO AGAIN
		RETLW		0				;DONE RETURN TO CALL


;********************RS232 TRANSMITTER INSTRUCTIONS***********

BIT_DELAY 
		MOVLW	D'200'			   	;GENERATING 1200 BAUD DELAY
		MOVWF	DELAY_CNTR         	;LOAD INTO COUNTER
	
LOOP    NOP						   	;TIMEING HELPER
	    DECFSZ	DELAY_CNTR,1		;DECREMENT TIMER
		GOTO	LOOP		       	;DECREASE COUNTER
        RETLW   0                   ;RETURN TO CALL 

SEND   	MOVLW	D'8'			   ;LOAD 8 INTO W EIGHT BITS IN A BYTE
	    MOVWF	BIT_CNTR           ;LOAD
	    MOVLW	D'8'			   ;LOAD 8 INTO W EIGHT BITS IN A BYTE
	    MOVWF	BIT_CNTR1		   ;LOAD
	    BCF	    SERIAL_OUT		   ;BRING SERIAL OUT LOW
        CALL    BIT_DELAY          ;START BIT SERIAL LOW FOR 1200 BAUD


XMIT    RRF     XMT_BYTE,1         ;ROTATE XMT BYTE FOR NEXT BIT
	    BTFSS	3,0				   ;CHECK TO SEE BIT SET
	    BCF		SERIAL_OUT		   ;NOT SET BRING SERIAL OUT LOW
	    BTFSC	3,0				   ;CHECK TO SEE IF SET	
	    BSF		SERIAL_OUT		   ;BIT SET MAKE SERIAL OUT HIGH
        CALL    BIT_DELAY          ;DATA BIT DELAY FOR 1200 BAUD
        DECFSZ 	BIT_CNTR,1		   ;CHECK TO SEE IT 8 BITS HAVE BEEN SENT
        GOTO	XMIT               ;DO UNTIL EIGHT BITS ARE SENT
 ;********************SECOND BYTE BEING SENT.  TOTAL 2 BYTES 16 BITS
XMIT1	RRF     XMT_BYTE1,1         ;ROTATE XMT BYTE FOR NEXT BIT
	    BTFSS	3,0					;CHECK TO SEE BIT SET
	    BCF		SERIAL_OUT			;NOT SET BRING SERIAL OUT LOW
	    BTFSC	3,0  				;CHECK TO SEE IF SET	
	    BSF		SERIAL_OUT 			;BIT SET MAKE SERIAL OUT HIGH
        CALL    BIT_DELAY          	;DATA BIT DELAY FOR 1200 BAUD  
        DECFSZ 	BIT_CNTR1,1  		;CHECK TO SEE IT 8 BITS HAVE BEEN SENT
        GOTO	XMIT1               ;GET EIGHT BITS        
		BSF		SERIAL_OUT			;SET SERIAL OUT HIGH FOR END OF TRANSMISSION
		CALL	BIT_DELAY 			;CALL BIT DELAY
        RETLW	0                   ;RETURN 


		  

		END
